#!/usr/bin/perl
##########################################################################
#  Add Footer script v1.2.9
#   2005-2009 Niversoft ides logicielles
#  Script by Nicolas Hatier
##########################################################################
#
#  Copy this script and the .footer files in a subdirectory of the Communigate
#  pro base folder, for instance /var/Communigate/Filters (create the subfolder
#  if  it doesn't exists). Customize the .footer files before using this script
#  (continue reading).
#
#  In CGPro, add this rule:
#
#   Header Field  is not X-Mimer-ProcessedID:*
#  ANY ROUTE     is     SMTP*
#  Return-Path   is     *@yourdomain.com
#   Human Generated
#
#  Action: Execute   [FILE][RETPATH][RCPT] /usr/bin/perl Filters/AddFooter.pl
#          Discard
#
#  The filter will add the footer and resubmit the mail via the PIPE module.
#  WARNING! This WILL broke any digital signature added to the message since the
#  message is modified AFTER the digital signature has been applied. So don't use
#  this script if you use digital signatures.
#
#  Add your signature[s] in .footer files that came with this script. Different
#  signatures format are needed for different MIME types. If you don't want a
#  signature to be added to a specific mime type, delete the corresponding
#  .footer file. You can add your own .footer files if you want a signature to
#  be included in a specific additional MIME type
#
#  This script is provided AS IS, without any guarantee.
#
#  Questions? info at niversoft dot com
#

use strict;

mkdir ("Submitted") unless (-d "Submitted");
my $InputStream;
my $NoBoundary = 1;


exit(main());

sub GoToBoundary
{
   my $HeaderInfo = shift;
   my $boundary = $HeaderInfo->{"b"};
   return "not_found" unless (length($boundary));

   while (my $Line = <$InputStream>)
   {
      if ($Line =~ /--\Q$boundary\E(--)?/)
      {
         return $1 eq "--" ? "found_end" : "found";
      }
   }
   return "not_found";
}

sub QPEncode($)
{
   my $str = $_[0];
   $str =~ s/=/=3D/g;
   for my $i (0..255)
   {
      if (($i != 10) and ($i != 13) and ($i != 32) and (($i < 33) or ($i > 126)))
      {
         my $reg = eval("qr/".chr($i)."/");
         my $rep = sprintf("=%X", $i);
         $str =~ s/$reg/$rep/g;
      }
   }
   $str =~ s/(.{1,72}) /$1=20=\n/g;
   $str =~ s/ $/=20/g;
   return $str;
}

sub SkipEnvelope
{
   my $Line = <$InputStream>;
   if ($Line !~ /^[A-Z] /)
   {
      seek($InputStream, 0, 0)
   }
   else
   {
      do { $Line = <$InputStream>; } while (length $Line > 2)
   }
}

sub GetHeaderInfo($)
{
   my $HeaderLine = "";
   $_[0]->{"q"} = 0;
   $_[0]->{"4"} = 0;
   $_[0]->{"a"} = 0;

   while (my $Line = <$InputStream>)
   {
      $Line =~ s/[\n]//g;
      if ($Line =~ /^[ \t]/)
      {
         $Line =~ s/^[ \t]*//;
         $HeaderLine .= " $Line";
      }
      else
      {
         if ($HeaderLine =~ /content-type ?: ?([^\s;\"]*)/i)
         {
            #print "$1\n";
            $_[0]->{"c"} = $1;
            if ($HeaderLine =~ /boundary ?= ?\"([^\"]*)\"/i)
            {
               $_[0]->{"b"} = $1;
            }
            elsif ($HeaderLine =~ /boundary ?= ?([^\s;\"]*)/i)
            {
               $_[0]->{"b"} = $1;
            }
         }
         if ($HeaderLine =~ /content-transfer-encoding ?: ?([^\s;\"]*)/i)
         {
            my $Cte = $1;
            $_[0]->{"q"} = $Cte =~ /quoted-printable/i;
            $_[0]->{"4"} = $Cte =~ /base64/i;
         }
         if ($HeaderLine =~ /content-disposition ?: ?(\w*)/i)
         {
            $_[0]->{"a"} = $1 =~ /attachment/i;
         }
         $HeaderLine = $Line;
      }
      last if ($Line !~ /[^\n]/);
   }
}

#c content-type
#b boundary
#q is quoted-printable
#b is base64
#a is attachment

sub GetParts
{
   my %Parts = ();
   my %Encoding = ();
   my $DoMultipartAlternative = 0;
   my $DoMultipartMixed = 0;
   my $DoCurrentPart = 0;
   my $HeaderInfo = {"c" => "text/plain"};
   GetHeaderInfo($HeaderInfo);

   while (1)
   {
      if ($DoCurrentPart)
      {
         while (1)
         {
            unless (length $HeaderInfo->{"b"})
            {
               while (my $Line = <$InputStream>) {};
            }
            my $FoundBoundary = GoToBoundary($HeaderInfo);

            if ($FoundBoundary ne "not_found")
            {
               $NoBoundary = 0;
            }
            unless ($HeaderInfo->{"4"} or $HeaderInfo->{"a"})
            {
               $Parts{$HeaderInfo->{"c"}} = tell($InputStream);
               $Encoding{$HeaderInfo->{"c"}} = $HeaderInfo->{"q"};
            }
            if ($FoundBoundary eq "found_end")
            {
               last;
            }

            my $OldBoundary = $HeaderInfo->{"b"};
            GetHeaderInfo($HeaderInfo);
            if ($DoMultipartAlternative and ($HeaderInfo->{"c"} eq "multipart/mixed"))
            {
               $DoMultipartAlternative = 0;
               last;
            }
            else
            {
               $HeaderInfo->{"b"} = $OldBoundary;
            }
            last if (eof($InputStream) or !length $HeaderInfo->{"c"});
         }
         last if (eof($InputStream));
      }

      if ($DoMultipartAlternative || ($HeaderInfo->{"c"} =~ m|^multipart/alternative|i))
      {
         $DoMultipartAlternative = 1;
         $DoCurrentPart = 1;

         last if (GoToBoundary($HeaderInfo) ne 'found');

         my $OldBoundary = $HeaderInfo->{"b"};
         GetHeaderInfo($HeaderInfo);
         $HeaderInfo->{"b"} = $OldBoundary;
      }
      elsif ($HeaderInfo->{"c"} =~ m|^multipart/|i)
      {
         $DoMultipartMixed = 1;
         last if (GoToBoundary($HeaderInfo) ne 'found');

         my $OldBoundary = $HeaderInfo->{"b"};
         GetHeaderInfo($HeaderInfo);
         $HeaderInfo->{"b"} = $OldBoundary unless ($HeaderInfo->{"c"} =~ m|^multipart/|i);
      }
      else
      {
         if ($HeaderInfo->{"c"} =~ m|^text/|i)
         {
            $DoCurrentPart = 1;
         }
         else
         {
            last if (GoToBoundary($HeaderInfo) ne 'found');
            my $NewHeaderInfo = {};
            GetHeaderInfo($NewHeaderInfo);
            my $OldBoundary = $HeaderInfo->{"b"};
            GetHeaderInfo($HeaderInfo);
            $HeaderInfo->{"b"} = $OldBoundary unless ($HeaderInfo->{"c"} =~ m|^multipart/|i);
         }
      }
   }
   return [\%Parts, \%Encoding];
}



sub main
{
   my %params = ( "f" => [], "r" => [], "p" => [] );
   my $param = "f";
   foreach my $arg (@ARGV)
   {
      if ($arg =~ /^-(.)$/)
      {
         $param = $1;
      }
      elsif (defined($params{$param}))
      {
         push @{$params{$param}}, $arg;
      }
   }

   open($InputStream, $params{"f"}[0]) || die ("can't open file $params{'f'}[0]");

   SkipEnvelope();

   my $Parts = GetParts();

   my $OutFile;
   my $processedid;
   do
   {
      $processedid = time.int(rand(10000000));
      $OutFile = "Submitted/outfile$processedid";
   }   while ((-f "$OutFile.tmp") || (-f "$OutFile.sub"));

   open OUTPUT, ">$OutFile.tmp";
   select OUTPUT;

   print "Return-Path: $params{'p'}[0]\n";
   print "Envelope-To: $_\n" foreach (@{$params{'r'}});
   print "X-Mimer-ProcessedID: $processedid\n";

   seek($InputStream, 0, 0);

   SkipEnvelope();

   unless (-t STDIN)
   {
      print $_ foreach(<STDIN>);
   }

   my %footers = ();

   my $rundir = $0;
   $rundir =~ s|\\|/|g;
   $rundir = index($rundir, "/")>=0 ? substr($rundir, 0, rindex($rundir, "/")) : ".";

   foreach my $key (keys %{$Parts->[0]})
   {
      my $mime_type = ($key eq "plain/text") ? "text/plain" : $key;
      $mime_type =~ s|/|-|;
      $mime_type = "$rundir/$mime_type.footer";
      if (-f $mime_type)
      {
         $footers{$Parts->[0]->{$key}} = [$mime_type, $Parts->[1]->{$key}];
      }
   }



   while (my $Line = <$InputStream>)
   {
      my $foot = $footers{tell($InputStream)};

      if (eof($InputStream) && $NoBoundary)
      {
         print $Line;
         $Line = "";
      }
      if ((defined $foot->[0]) && (open (FOOTER, $foot->[0])))
      {
         my $footText = join("", <FOOTER>);
         if ($foot->[1])
         {
            $footText = QPEncode($footText);
         }

         print "$footText\n";
      }
      print $Line;
   }

   close(select(STDOUT));
   rename ("$OutFile.tmp", "$OutFile.sub");
   return 0;
}
